#include "stm32f10x.h"

// unmask interrupts
#define EnableMaple_nDLatch()	EXTI->IMR	|= 0x01
#define EnableMaple_nSTRCV()	EXTI->IMR	|= 0x02
#define EnableMaple_nRST()	EXTI->IMR	|= 0x04
#define EnableMapleRxD()	EXTI->IMR	|= 0x08
#define EnableMaple_nEOP()	EXTI->IMR	|= 0x40
// mask interrupts, and mask pending flags
#define DisableMaple_nDLatch()	{EXTI->IMR	&= ~0x01; EXTI->PR	|= 0x01;}
#define DisableMaple_nSTRCV()	{EXTI->IMR	&= ~0x02; EXTI->PR	|= 0x02;}
#define DisableMaple_nRST()	{EXTI->IMR	&= ~0x04; EXTI->PR	|= 0x04;}
#define DisableMapleRxD()	{EXTI->IMR	&= ~0x08; EXTI->PR	|= 0x08;}
#define DisableMaple_nEOP()	{EXTI->IMR	&= ~0x40; EXTI->PR	|= 0x40;}

#define DisableMapleINTs()	{EXTI->IMR	&= ~0x6F; EXTI->PR	|= 0x6F;}
#define DisableMapleINTsExceptRST()	{EXTI->IMR	&= ~0x6B; EXTI->PR	|= 0x6B;}

/*Define CPLD MIE Companion Pins*/
extern unsigned char m_wPortState;

#define	nDLatch			(GPIO_Pin_0)	//Data latched for read, DATA LATCH on Negative pulse
#define	nSTRCV			(GPIO_Pin_1)	//Receive start on positive pulse
#define	nRST_OCPY		(GPIO_Pin_2)	//Device hard reset from host or occupancy pattern receive (See State of XMAPLE IO0 = 1 Occupancy received, only in XMAPLE Ver 1.1 or higher), RESET = 0
#define	RxD					(GPIO_Pin_3)	//Receive in progress, RECEIVE = 1
#define	OCPY				(0)						//OCPY pin present in XMAPLE Ver 1.1 but not connected to CPU 
#define	INHRxD			(GPIO_Pin_4)	//Inhibit MIE Companion signals detection, INHIBIT = 1
#define	FERR				(GPIO_Pin_5)	//Frame Error, ERROR = 1
#define	EOP					(GPIO_Pin_6)	//End of packet detection, ERROR = 1

#define IsMIERxD() 				(GPIOA->IDR & RxD)
#define GetReceiveStart() (GPIOA->IDR & nSTRCV)
#define GetDataLatch() 		(GPIOA->IDR & nDLatch)

#define EnMIECOMP()		GPIOA->BRR = INHRxD
#define DisMIECOMP()	GPIOA->BSRR = INHRxD

/*PC8 = SDCKA*/
#define SDCKA_IN()	GPIOC->CRH = ((GPIOC->CRH & 0xFFFFFFF0) | 0x08)
/*10 MHz*/
#define SDCKA_OUT()	GPIOC->CRH = ((GPIOC->CRH & 0xFFFFFFF0) | 0x01)
#define SDCKA_SET(A)	GPIOC->BSRR = ((A) ? 0x0100 : 0x01000000)
#define SDCKA_GET()	(GPIOC->IDR & 0x0100)
/*PC9 = SDCKB*/
#define SDCKB_IN()	GPIOC->CRH = ((GPIOC->CRH & 0xFFFFFF0F) | 0x80)
/*10 MHz*/
#define SDCKB_OUT()	GPIOC->CRH = ((GPIOC->CRH & 0xFFFFFF0F) | 0x10)
#define SDCKB_SET(B)	GPIOC->BSRR = ((B) ? 0x0200 : 0x02000000)
#define SDCKB_GET()	(GPIOC->IDR & 0x0200)

/*Receive XMAPLE Data Port State*/
#define RXMDState() (GPIOC->IDR & 0xFF)

/* ON MACRO for LEDs */
#define	LED_MTRM_ON()		GPIOA->BSRR = LED_MTRM
#define	LED_MRCV_ON()		GPIOA->BSRR = LED_MRCV
#define	LED_RD_ON()			GPIOC->BSRR = LED_RD
#define	LED_WR_ON()			GPIOC->BSRR = LED_WR

/* OFF MACRO for LEDs */
#define	LED_MTRM_OFF()		GPIOA->BRR = LED_MTRM
#define	LED_MRCV_OFF()		GPIOA->BRR = LED_MRCV
#define	LED_RD_OFF()			GPIOC->BRR = LED_RD
#define	LED_WR_OFF()			GPIOC->BRR = LED_WR

/*Define Other Pins*/
/*GPIOA*/
#define	USBCON			(GPIO_Pin_7)
#define	LED_MRCV		(GPIO_Pin_9)	//LED Maple BUS Receive
#define	LED_MTRM		(GPIO_Pin_10)	//LED Maple BUS Transmit
#define	USBDM				(GPIO_Pin_11)
#define	USBDP				(GPIO_Pin_12)
#define	USBBOOT			(GPIO_Pin_15)
/*GPIOB*/
/*External IO 8 bit MCU BUS*/
#define	IO0					(GPIO_Pin_0)
#define	IO1					(GPIO_Pin_1)
#define	IO2					(GPIO_Pin_2)
#define	IO3					(GPIO_Pin_3)
#define	IO4					(GPIO_Pin_4)
#define	IO5					(GPIO_Pin_5)
#define	IO6					(GPIO_Pin_6)
#define	IO7					(GPIO_Pin_7)
#define IO07				(IO0 | IO1 | IO2 | IO3 | IO4 | IO5 | IO6 | IO7)
/*GPIOC*/
/*XMAPLE I 8 bit BUS*/
#define	I0					(GPIO_Pin_0)
#define	I1					(GPIO_Pin_1)
#define	I2					(GPIO_Pin_2)
#define	I3					(GPIO_Pin_3)
#define	I4					(GPIO_Pin_4)
#define	I5					(GPIO_Pin_5)
#define	I6					(GPIO_Pin_6)
#define	I7					(GPIO_Pin_7)
#define	I07					(I0 | I1 | I2 | I3 | I4 | I5 | I6 | I7)
/*MCU Pins*/
#define	SDCKA				(GPIO_Pin_8)	//MCU Transmit Line A
#define	SDCKB				(GPIO_Pin_9)	//MCU Transmit Line B
#define	LED_RD			(GPIO_Pin_10)	//LED ...performing Read access
#define	LED_WR			(GPIO_Pin_11)	//LED ...performing Write access
#define	STDI				(GPIO_Pin_12)	//XMAPLE JTAG Test Data In
#define	STMS				(GPIO_Pin_13)	//XMAPLE JTAG Test Mode Select
#define	STCK				(GPIO_Pin_14)	//XMAPLE JTAG Test Clock
#define	STDO				(GPIO_Pin_15)	//XMAPLE JTAG Test Data Out
/*GPIOD*/
#define	nGRST				(GPIO_Pin_2)	//Mie Componets logic reset, RESET = 0

/*XMAPLE Version Detect Patterns*/
#define PAT_DETA		0x55
#define PAT_DETB		0xAA

/*BOARD Initialization Defines*/
#define INIT_RCC			0x01	//Initialize APB CLOCKs
#define INIT_GPIO			0x02	//Initialize GPIO Directions and states
#define INIT_XMCLK		0x04	//Initialize PA8 Output clock for XMAPLE 
#define INIT_XMEXT		0x08	//Initialize Receive EXT Interrupts for XMAPLE Module

/* Some Timer Defines */
#define POOL_PERIOD_VALUE			200
#define POOL_FRAME_TIMESHIFT	200

#define	CEN		0x0001
#define UDIS	0x0002
#define OPM		0x0008

#define TIMEOUT_TIMER	TIM1

/*Initialization*/
void InitBOARD(int iFlags);
/*Special XMAPLE Functions*/
unsigned char GetXMVer(void);
/*Secifical STM32F functions*/
unsigned int isInterrupt(void);
void POOL_TIMER_Init(TIM_TypeDef* TIMx);
void ChargePOOLTimer(TIM_TypeDef* TIMx, unsigned short wTimeOut);
unsigned int isTimeoutEnds(TIM_TypeDef* TIMx);
