#if !defined (MAPLEBUS)
#define MAPLEBUS

#include <stdbool.h>

extern unsigned char DeviceAddress;
extern char *sRecErrorCodes[];

/* If our conmtroller is LSB we must convert it to MSB */
#define MAKE_DWORD(DW) ((DW >> 24) | ((DW << 24)) | ((DW << 8) & 0xFF0000) | ((DW >> 8) & 0xFF00))
#define MAKE_WORD(W) 	(((W >> 8) & 0xFF) | ((W << 8) & 0xFF00))

//Command Codes
//HOST
#define	DeviceRequest				0x01
#define	AllStatusRequest		0x02
#define	DeviceReset					0x03
#define	DeviceKill					0x04
#define	GetCondition				0x09
#define	GetMediaInfo				0x0A
#define	BlockRead						0x0B
#define	BlockWrite					0x0C
#define	GetLastError				0x0D
#define	SetCondition				0x0E
#define	FT4Control					0x0F
#define	ARControl						0x10
#define	TransmitAgain				0xFC
//Device
#define	DeviceStatus				0x05
#define	DeviceAllStatus			0x06
#define	DeviceReply					0x07
#define	DataTransfer				0x08
#define	ARError							0xF9
#define	LCDError						0xFA
#define	FileError						0xFB
#define	TransmitAgain				0xFC
#define	CommandUnknown			0xFD
#define	FunctionTypeUnknown	0xFE

/*Receive frame error codes*/
#define	RCV_OK							(uint8_t)0x00	//Receive OK
#define	RCV_CRC_ERROR				0x01	//CRC Detection error while receive data
#define	RCV_OVF_ERROR				0x02	//Receive Buffer Overfflow
#define	RCV_TIME_OUT				0x03	//Timeout exceeded while receive data wait
#define	RCV_NO_RXD					0x04	//Packet receive signalled, but START Pattern not detected
#define	RCV_FERR						0x05	//Frame Error

/*Host PORTS*/
#define AP_PORT_MASK				0xC0		//Extraction HOST PORT ID Mask

#define AP_PORTA						0x00		//HOST PORT 'A'
#define AP_PORTB						0x40		//HOST PORT 'B'
#define AP_PORTC						0x80		//HOST PORT 'C'
#define AP_PORTD						0xC0		//HOST PORT 'D'

/*Devices On Host PORTS*/
#define AP_DEVICE						0x20

#define AP_DEVICEA					(AP_PORTA | AP_DEVICE)		//Device on HOST PORT 'A'
#define AP_DEVICEB					(AP_PORTB | AP_DEVICE)		//Device on HOST PORT 'B'
#define AP_DEVICEC					(AP_PORTC | AP_DEVICE)		//Device on HOST PORT 'C'
#define AP_DEVICED					(AP_PORTD | AP_DEVICE)		//Device on HOST PORT 'D'

/*Expansion device present detection bits*/
#define	EXP_DEV1						0x01	//Expansion Device in Device Port '1'
#define	EXP_DEV2						0x02	//Expansion Device in Device Port '2'
#define	EXP_DEV3						0x04	//Expansion Device in Device Port '3'
#define	EXP_DEV4						0x08	//Expansion Device in Device Port '4'
#define	EXP_DEV5						0x10	//Expansion Device in Device Port '5'

#define MAX_EXP_DEV					0x05	//Maximum aviable devices on one maple BUS

/*Device functions*/
#define CONTROLLER	MAKE_DWORD(0x00000001)		//FT0 : Controller Function
#define STORAGE			MAKE_DWORD(0x00000002)		//FT1 : Storage Function
#define LCD					MAKE_DWORD(0x00000004)		//FT2 : B/W LCD Function
#define TIMER				MAKE_DWORD(0x00000008)		//FT3 : Timer Function
#define AUDIO_INPUT	MAKE_DWORD(0x00000010)		//FT4 : Audio input device Function
#define AR_GUN			MAKE_DWORD(0x00000020)		//FT5 : AR-Gun Function
#define KEYBOARD		MAKE_DWORD(0x00000040)		//FT6 : Keyboard
#define GUN					MAKE_DWORD((unsigned int)0x00000080)		//FT7 : Light-Gun Function
#define VIBRATION		MAKE_DWORD((unsigned int)0x00000100)		//FT8 : Vibration Function
#define MOUSE				MAKE_DWORD((unsigned int)0x00000200)		//FT9 : Pointing Function
#define EXMEDIA			MAKE_DWORD((unsigned int)0x00000400)		//FT10 : Exchange Media Function
#define CAMERA			MAKE_DWORD((unsigned int)0x00000800)		//FT11 : Camera Device Functio

#define ALL_DEVICES	0x00000FFF		//FT0 - FT11 Devices Mask

/*Functions definition 'CONTROLLER'*/
#define FD_CONT_BT_Ra			(1 << 7)
#define FD_CONT_BT_La			(1 << 6)
#define FD_CONT_BT_Da			(1 << 5)
#define FD_CONT_BT_Ua			(1 << 4)
#define FD_CONT_BT_Start	(1 << 3)
#define FD_CONT_BT_A			(1 << 2)
#define FD_CONT_BT_B			(1 << 1)
#define FD_CONT_BT_C			(1 << 0)
#define FD_CONT_BT_Rb			(1 << 15)
#define FD_CONT_BT_Lb			(1 << 14)
#define FD_CONT_BT_Db			(1 << 13)
#define FD_CONT_BT_Ub			(1 << 12)
#define FD_CONT_BT_D			(1 << 11)
#define FD_CONT_BT_X			(1 << 10)
#define FD_CONT_BT_Y			(1 << 9)
#define FD_CONT_BT_Z			(1 << 8)

#define FD_CONT_axis_1		(1 << 16)
#define FD_CONT_axis_2		(1 << 17)
#define FD_CONT_axis_3		(1 << 18)
#define FD_CONT_axis_4		(1 << 19)
#define FD_CONT_axis_5		(1 << 20)
#define FD_CONT_axis_6		(1 << 21)

/*Functions definition 'STORRAGE'*/
#define FD_STOR_REMOVABLE	0x80			//This sets whether the media on which the data is stored is removable or not.  
#define FD_STOR_CRC				0x40			//This sets whether the CRC calculation (16 bits) is required or not for block data reads and writes.

/*Functions definition 'LCD'*/
#define FD_LCD_HV1		0x80				// Specifies whether the LCD data rows 
#define FD_LCD_HV0		0x40				//are horizontal or vertical.
#define FD_LCD_BW			0x20				// Specifies Normally Black or Normally White. Normally	B/W White	= 0, Black = 1

/*Functions definition 'TIMER'*/
#define FD_TIM_DOF		0x01			//Day of the week
#define FD_TIM_SEC		0x02			//Second
#define FD_TIM_MIN		0x04			//Minute
#define FD_TIM_HOR		0x08			//Hour
#define FD_TIM_DAY		0x10			//Day
#define FD_TIM_MON		0x20			//Month
#define FD_TIM_YER		0x40			//Year

#define FD_TIM_U			0x01			//Indicates the "Up" button on the cross-shaped directional button.
#define FD_TIM_D			0x02			//Indicates the "Down" button on the cross-shaped directional button.
#define FD_TIM_L			0x04			//Indicates the "Left" button on the cross-shaped directional button.
#define FD_TIM_R			0x08			//Indicates the "Right" button on the cross-shaped directional button.
#define FD_TIM_A			0x10			//Indicates the "A" (example function: Enter, etc.) button.
#define FD_TIM_B			0x20			//Indicates the "B" (example function: Cancel, etc.) button.
#define FD_TIM_C			0x40			//Indicates the "C" (example function: Menu, etc.) button.
#define FD_TIM_S			0x80			//Indicates the "Start" (example function: Pause, etc.) button.

/*Alarm flags*/
#define FD_TIM_AL0		0x40			// This indicates whether an alarm 
#define FD_TIM_AL1		0x80			//is on or not.

/*Receive next frame time out*/
#define RCV_TIME_OUT_CNT		0x20 /* 32x1ms */

/*Defines for MapleSend*/
#define	TIMEOUT_CHECK			0x01
#define NO_TIMEOUT_CHECK	0x00

#define	NO_TIMEOUT_EXCEEDED	0x01
#define TRM_OK							0x00

/* STORAGE Structs */
typedef struct tagSTORAGE_INFO
{
	unsigned short wTotalSize;
	unsigned short wPartitionNo;
	unsigned short wSystemAreaBlockNo;
	unsigned short wFATAreaBlockNo;
	unsigned short wNumberOfFATAreaBlocks;
	unsigned short wFileInformationBlockNo;
	unsigned short wNumberOfFileInformationBlocks;
	unsigned char bReserved;
	unsigned char bVolumeICON;
	unsigned short wSaveAreaBlockNo;
	unsigned short wNumberOfSaveAreaBlocks;
}STORAGE_INFO, *LPSTORAGE_INFO;

typedef struct tagMapleDataFrame
{
/*Define counters for SOP and EOP Detect*/
	unsigned char Command;
	unsigned char DestAP;
	unsigned char OriginAP;
	unsigned char DataSize;
	 union{
			unsigned char bLine[1020];
			unsigned int dwLine[255];
	 }Data;
	unsigned char stub[4];		// CRC and Stub Frame
}MAPLEFRAME, *LPMAPLEFRAME;

extern volatile unsigned char m_bRet;

#define GetLastReceiveError() m_bRet

extern MAPLEFRAME 		inMapleFrame;		//Input Maple Frame for Receive
extern MAPLEFRAME 		outMapleFrame;		//Output Maple Frame for Transmit

//Current device Peripheral Function
extern unsigned long m_dwPeripheralFunction;
//Our device address
extern unsigned char m_bOurDeviceAddress;
//Number of Transmits again
extern unsigned char m_bNumbersOfTramsmits;
//Last Sended command
extern unsigned char m_bPreviousCommand;
//Last Sended Data Size
extern unsigned char m_bPreviousDataSize;

/*Reset Device*/
void ResetDevice(void);
/*Send by Mapple Protocol*/
unsigned char MapleSend(LPMAPLEFRAME MapleFrame, unsigned int boolCheckTimeout);
/*Receive Frame by Mapple Protocol by Interrupts*/
unsigned char MapleReceive(LPMAPLEFRAME MapleFrame);
/*Extract from DeviceInformation struct Peripheral Function Type, defined by user*/
void GetPeripheralFunctionType(void);
/*Process frame then use MIELIB for device build*/
void ProcessDeviceFrame(LPMAPLEFRAME MPFrame);
/* Handling Receive Clock */
void EXTI0_IRQHandler(void);

/*Functions for processing while host*/
unsigned char FuncDeviceRequest(unsigned char APAddress);
unsigned char GetStorageInfo(unsigned char APAddress, LPSTORAGE_INFO lpsSInf);
unsigned short OneBlockRead(unsigned char APAddress, unsigned char *BlockData, unsigned short wBlockNumber, unsigned short wFuncType);
unsigned char FramesBlockWrite(unsigned char APAddress, unsigned short wBlockNumber, unsigned short wFuncType, unsigned char *lpbBuffer);
unsigned char OneBlockWrite(unsigned char APAddress, unsigned short wBlockNumber, unsigned short wFuncType, unsigned char *lpbBuffer, int iBlockSize);


#endif
