#if !defined (MAPLE_DEVICES)
#define MAPLE_DEVICES

#include "maplebus.h"

//Information Packet
typedef struct tagDEVICEINFORMATION{
	unsigned long FT0;
	unsigned long FT1;
	unsigned long FT2;
	unsigned long FT3;
	unsigned char Country; 
	unsigned char Connection; 
	char Name[30];
	char License[60];
	unsigned int Standby;
	unsigned int Maximum;
}DEVICEINFORMATION,*LPDEVICEINFORMATION;

//*******************************************
//Mouse implementation segment
//*******************************************
#define	AOV0		0x0001
#define	AOV1		0x0002
#define	AOV2		0x0004

#define	MW_BUTTON	0x0008
#define	MA_BUTTON	0x0004
#define	MB_BUTTON	0x0002

#define	DEF_BUTTON	0x00F1

#define	ST_CONSTANT	0x0200

typedef struct tagMOUSECONDITION
{
	unsigned long FunctionType;
	unsigned int BTN;	//Operating button data 
	unsigned int AOV;	//Analog coordinate overflow 
	unsigned int AC1;	//X-axis analog coordinate region
	unsigned int AC2;	//Y-axis analog coordinate region 
	unsigned int AC3;	//Z-axis analog coordinate region (wheel) 
	unsigned int AC4;	//Constant (00h-02h, fixed) 
	unsigned int AC5;	//Constant (00h-02h, fixed) 
	unsigned int AC6;	//Constant (00h-02h, fixed) 
	unsigned int AC7;	//Constant (00h-02h, fixed) 
	unsigned int AC8;	//Constant (00h-02h, fixed) 
}MOUSECONDITION,*LPMOUSECONDITION;

//*******************************************
//Keybort implementation segment
//*******************************************
typedef struct tagKEYBOARDCONDITION
{
	unsigned long FunctionType;
	unsigned char ChangeKeyBit;
	unsigned char LEDInformation;
	unsigned char KeyScanCodeArray[6];
}KEYBOARDCONDITION,*LPKEYBOARDCONDITION;

//*******************************************
// Controller implementation segment
//*******************************************

/* Report first byte */
#define	CRa_BUTTON		0x80
#define	CLa_BUTTON		0x40
#define	CDa_BUTTON		0x20
#define	CUa_BUTTON		0x10
#define	CST_BUTTON		0x08
#define	CA_BUTTON			0x04
#define	CB_BUTTON			0x02
#define	CC_BUTTON			0x01
/* Report second byte */
#define	CX_BUTTON			0x04
#define	CY_BUTTON			0x02
#define	CZ_BUTTON			0x01

typedef struct tagCONTROLLERCONDITION
{
	unsigned long FunctionType;
	unsigned char ButtonsLO;
	unsigned char ButtonsHI;
	unsigned char AR_axis;
	unsigned char AL_axis;
	unsigned char Xa_axis;
	unsigned char Ya_axis;
	unsigned char Not_used_A;	//Must be 0x80
	unsigned char Not_used_B;	//ust be 0x80
}CONTROLLERCONDITION,*LPCONTROLLERCONDITION;


void GetPeripheralFunctionType(void);
void ProcessDeviceFrame(LPMAPLEFRAME MapleFrame);
unsigned char GetDeviceCondition(void *DeviceCond);

unsigned char MakeOurDeviceAddress(unsigned char DeviceAddress);
unsigned char GetMouseCondition(LPMOUSECONDITION MouseCond);
unsigned char GetControllerCondition(LPCONTROLLERCONDITION ContCond);
unsigned char GetKeyboardCondition(LPKEYBOARDCONDITION ContCond);

#endif
