#if !defined (DEVICE_TYPES)
#define DEVICE_TYPES

/*File Errors, Storage Function*/
#define	PTError							0x00000001	//The specified partition does not exist.
#define	PhaseError					0x00000002	//The Phase + 1 value is greater than the value of RA or WA
#define	BlockError					0x00000004	//The specified block does not exist.
#define	PhysicalWriteError	0x00000008	//The data was not written in the storage device (media) correctly.
#define	LengthError					0x00000010	//The specified data size does not match the size of the data that was sent.
#define	DomainError					0x00000020	//An access was made to an access-prohibited area.
#define	DriveNotReady				0x00000040	//Removable media was not mounted.
#define	CRCError						0x00000080	//While the CRC flag was set to "1," a CRC error occurred while receiving block data.
#define	UndefinedError			0x80000000	//An undefined error occurred.

#if !defined (MAPLE_HOST)

//Setup library as anly one device
//#define EN_CONTROLLER
//#define EN_KEYBOARD
//#define EN_GUN
//#define EN_MOUSE

#if !defined (EN_CONTROLLER) && !defined (EN_KEYBOARD) &&  !defined (EN_GUN) &&  !defined (EN_MOUSE)
		#error "Device type must be defined!!!"
#endif

#if defined (EN_MOUSE)
	#if (defined (EN_KEYBOARD) ||  defined (EN_GUN) ||  defined (EN_CONTROLLER))
		#error "Only one base type device type must be defined!!!"
	#endif

const static unsigned char DeviceInformation[] ={
	//Device ID
	0x00, 0x00, 0x02, 0x00, 
	0x00, 0x0E, 0x07, 0x00, 
	0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00,
	//Country specification 
	0xFF,
	//Connection method 
	0x00,
		//Model name 30 BYTES
		'D','r','e','a','m','c','a','s','t','M',
		'o','u','s','e',' ',' ',' ',' ',' ',' ',
		' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
		//License 60 BYTES
		'P','r','o','d','u','c','e','d',' ','B',
		'y',' ','o','r',' ','U','n','d','e','r',
		' ','L','i','c','e','n','s','e',' ','F',
		'r','o','m',' ','S','E','G','A',' ','E',
		'N','T','E','R','P','R','I','S','E','S',
		',','L','T','D','.',' ',' ',' ',' ',' ',
		//Standby current consumption - 40 ma
		0x01,0x90,
		//Maximum current consumption - 50 ma
		0x01,0xF4
};

const static unsigned char DeviceRevisionInfo[] = "Version 1.000, 2021/01/03, SMALL MIE INTERFACE+STM32F103R6T6 MODULE-6125-AB ";

//Free device status
//Unless there are special reasons for not doing so, the following 40 bytes of information should first be entered. 

//Versions (ROM, logic, etc.)  :13 bytes  E.g., Version 1.000 
//Recommended: The version orthography as shown in the example, three digits following the decimal 
//point, and the first release is 1.000. 
//Release date  :10 bytes  E.g., 1998/05/11 
//Recommended: Year in four digits, month in two digits, and day in two digits. 
//IC parts number  :14 bytes  E.g., 315-6125-AB   
//Recommended: Enter space (20h) in the empty area. 
//Each item should be separated by ,. 
//Example: Version 1.000,1998/05/11,315-6125-AB  , 

const static unsigned char FreeStatus[] = "Version 1.000,2021/01/03,STM32F1-6125-AB,";

#endif

#if defined (EN_CONTROLLER)
	#if (defined (EN_KEYBOARD) ||  defined (EN_GUN) ||  defined (EN_MOUSE))
		#error "Only one base type device type must be defined!!!"
	#endif

const static unsigned char DeviceInformation[] ={
	//Device ID
	0x00,0x00,0x00,0x01,
		0x00,0x0F,0x06,0xFE,
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,
		//Country specification 
		0xFF,
		//Connection method 
		0x00,
		//Model name 30 BYTES
		'D','r','e','a','m','c','a','s','t',' ',
		'C','o','n','t','r','o','l','l','e','r',
		' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
		//License 60 BYTES
		'P','r','o','d','u','c','e','d',' ','B',
		'y',' ','o','r',' ','U','n','d','e','r',
		' ','L','i','c','e','n','s','e',' ','F',
		'r','o','m',' ','S','E','G','A',' ','E',
		'N','T','E','R','P','R','I','S','E','S',
		',','L','T','D','.',' ',' ',' ',' ',' ',
		//Standby current consumption - 43 ma
		0xAE,0x01,
		//Maximum current consumption - 50 ma
		0xF4,0x01
};

const static unsigned char DeviceRevisionInfo[] = "Version 1.000, 2021/01/03, SMALL MIE INTERFACE+STM32F103R6T6 MODULE-6125-AB ";

//Free device status
//Unless there are special reasons for not doing so, the following 40 bytes of information should first be entered. 

//Versions (ROM, logic, etc.)  :13 bytes  E.g., Version 1.000 
//Recommended: The version orthography as shown in the example, three digits following the decimal 
//point, and the first release is 1.000. 
//Release date  :10 bytes  E.g., 1998/05/11 
//Recommended: Year in four digits, month in two digits, and day in two digits. 
//IC parts number  :14 bytes  E.g., 315-6125-AB   
//Recommended: Enter space (20h) in the empty area. 
//Each item should be separated by ,. 
//Example: Version 1.000,1998/05/11,315-6125-AB  , 

const static unsigned char FreeStatus[] = "Version 1.000,2021/01/03,STM32F1-6125-AB,";

#endif

#if defined (EN_KEYBOARD) 
	#if (defined (EN_CONTROLLER) ||  defined (EN_GUN) ||  defined (EN_MOUSE))
		#error "Only one device type must be defined!!!"
	#endif
	
const static unsigned char DeviceInformation[] ={
//Device ID
0x00,0x00,0x00,0x40,
0x0E,0x04,0x00,0x80,
0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,
//Country specification 
0xFF,
//Connection method 
0x00,
//Model name 30 BYTES
		'1','0','2','k','e','y',' ','K','e','y',
		'b','o','a','r','d',' ','f','o','r',' ',
		'E','U','R',' ',' ',' ',' ',' ',' ',' ',
//License 60 BYTES
		'P','r','o','d','u','c','e','d',' ','B',
		'y',' ','o','r',' ','U','n','d','e','r',
		' ','L','i','c','e','n','s','e',' ','F',
		'r','o','m',' ','S','E','G','A',' ','E',
		'N','T','E','R','P','R','I','S','E','S',
		',','L','T','D','.',' ',' ',' ',' ',' ',
//Standby current consumption - 43 ma
0x01,0xAE,
//Maximum current consumption - 50 ma
0x01,0xF4
};

//Free device status is exported from controller
const static unsigned char FreeStatus[] = "Version 1.000,2021/01/03,STM32F1-6125-AB,";

#endif

#if defined (EN_GUN) && (defined (EN_KEYBOARD) ||  defined (EN_CONTROLLER) ||  defined (EN_MOUSE))
		#error "Only one device type must be defined!!!"
#endif

#if defined (EN_MOUSE) && (defined (EN_KEYBOARD) ||  defined (EN_GUN) ||  defined (EN_CONTROLLER))
		#error "Only one device type must be defined!!!"
#endif

#endif	/*MAPLE_HOST*/

#endif /*DEVICE_TYPES*/
