#include "../inc/Interfaces.h"
#include "../inc/Board.h"
#include "..\inc\delay.h"  

/*
PIN				NAME				TYPE		DESCRIPTION
PA0				nDLatch			I				Device hardware reset from host, RESET = 0
PA1				nSTRCV			I				Receive start on positive pulse
PA2				nRST				I				Device hardware reset from host, RESET = 0
PA3				RxD					I				Receive in progress, RECEIVE = 1
Px?				OCPY				I				Receive occupancy pattern, OCPY = 0
PA4				INHRxD			I				Inhibit MIE Companion signals detection, INHIBIT = 1
PA5				FERR				I				Frame Error, ERROR = 1
PA6				EOP					I				End of packet detection, ERROR = 1
PA7				USBCON			O				Connect USB Bus, CONNECT = 0
PA8				GCLK				A				Clock for MIE Component
PA9				LED_MRCV		O				Maple BUS Receive
PA10			LED_MTRM		O				Maple BUS Transmit

PA11			USBDM				A				USB Data Minus
PA12			USBDP				A				USB Data Plus

PA13			SWDIO				A				DEBUG I/O
PA14			SWCLK				A				DEBUG Clock

PA15			USBBOOT			I				Bootloader BUTTON

PC[0..7]	D[0..7]			I				8 Bit data bus from MIE Component

PC8				SDCKA				I/O			MCU Transmit Line A
PC9				SDCKB				I/O			MCU Transmit Line B
PC10			LED_RD			O				...performing Read access
PC11			LED_WR			O				...performing Write access
PC12			STDI				O				XMAPLE JTAG Test Data In
PC13			STMS				O				XMAPLE JTAG Test Mode Select
PC14			STCK				O				XMAPLE JTAG Test Clock
PC15			STDO				I				XMAPLE JTAG Test Data Out

PB[0..7]	GPIO[0..7]	I/O			External IO 8 bit MCU BUS

PD2				nGRST				O				Mie Componets logic reset, RESET = 0

*/

/*************************************************************
**									BOARD CONFIGURATION											**
**************************************************************/

/*Enable RCC Clocks*/
void RCC_Configuration(void)
{
  /* GPIOA, GPIOB, GPIOC clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD, ENABLE);
}

/*Set default GPIO Configuration*/
void GPIO_Configuration(void)
{
	GPIO_InitTypeDef   GPIO_InitStructure;

	/* set MCU debugger to JTAG-DP Disabled and SW-DP Enabled (Free Pin(s): PA15, PB3*, PB4) */
	GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable , ENABLE);

	/* Set DEFAULT State for GPIOA Pins */
	GPIO_WriteBit(GPIOA, nDLatch | nSTRCV | nRST_OCPY | RxD | OCPY | INHRxD | FERR |	EOP | USBBOOT, Bit_SET);
	GPIO_WriteBit(GPIOA, LED_MRCV | LED_MTRM, Bit_RESET);
	
	/* Set DEFAULT State for GPIOB Pins */
	GPIO_WriteBit(GPIOB, IO07, Bit_RESET);
	
	/* Set DEFAULT State for GPIOC Pins */
	GPIO_WriteBit(GPIOC, SDCKA | SDCKB | STDO, Bit_SET);//STDI | STMS | STCK | 
	GPIO_WriteBit(GPIOC, I07 | LED_RD | LED_WR, Bit_RESET);
	
	/* Set DEFAULT State for GPIOD Pins */
	GPIO_WriteBit(GPIOD, nGRST, Bit_SET);
	
  /* Configure GPIOA pins as input */
  GPIO_InitStructure.GPIO_Pin = nDLatch | nSTRCV | nRST_OCPY | RxD | OCPY | INHRxD | FERR |	EOP | USBBOOT;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	/* Configure GPIOB pins as input */
  GPIO_InitStructure.GPIO_Pin = (IO07);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

	/* Configure GPIOC pins as input */
  GPIO_InitStructure.GPIO_Pin = (I07 | SDCKA | SDCKB | STDO);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  /* Configure GPIOA pins as output */
  GPIO_InitStructure.GPIO_Pin = LED_MRCV | LED_MTRM;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Configure GPIOB NONE */

  /* Configure GPIOC pins as output */
  GPIO_InitStructure.GPIO_Pin = LED_RD | LED_WR | STDI | STMS | STCK;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  /* Configure GPIOD pins as output */
  GPIO_InitStructure.GPIO_Pin = nGRST;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);
}

/*Setup EXTERNAL Interrupts*/
void InitEXTINT(void)
{
	EXTI_InitTypeDef   EXTI_InitStructure;
	NVIC_InitTypeDef   NVIC_InitStructure;

  /* Configure EXTI0 - nDLatch line */
  EXTI_InitStructure.EXTI_Line = EXTI_Line0;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;  
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);

  /* Configure EXTI1 - nSTRCV line */
  EXTI_InitStructure.EXTI_Line = EXTI_Line1;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;  
  EXTI_Init(&EXTI_InitStructure);

  /* Configure EXTI2 - nRST line */
  EXTI_InitStructure.EXTI_Line = EXTI_Line2;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;  
  EXTI_Init(&EXTI_InitStructure);
	
  /* Configure EXTI3 - RxD line */
  EXTI_InitStructure.EXTI_Line = EXTI_Line3;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;  
  EXTI_Init(&EXTI_InitStructure);
	
  /* Configure EXTI6 - EOP line */
  EXTI_InitStructure.EXTI_Line = EXTI_Line6;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;  
  EXTI_Init(&EXTI_InitStructure);
	
  /* Enable and set EXTI0 - nDLatch Interrupt priority */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x00;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x02;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable and set EXTI1 - nSTRCV Interrupt priority */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI1_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x03;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable and set EXTI2 - nRST Interrupt priority */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x03;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable and set EXTI3 - RxD Interrupt priority */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI3_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x03;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable and set EXTI6 - EOP Interrupt priority */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x03;
  NVIC_Init(&NVIC_InitStructure);
}

/*Setup OUTPUT Clocks for XMAPLE CPLD*/
void InitXMAPLEClk(void) 
{ 
    GPIO_InitTypeDef is; 
    
    is.GPIO_Pin = GPIO_Pin_8; 
    is.GPIO_Speed = GPIO_Speed_50MHz; 
    is.GPIO_Mode = GPIO_Mode_AF_PP; 
    GPIO_Init(GPIOA,&is); 
    
    RCC_MCOConfig(RCC_MCO_PLLCLK_Div2/*RCC_MCO_SYSCLK*/); 
    return; 
}

/*Whole XMAPLE-EB Init*/
void InitBOARD(int iFlags)
{
	if(iFlags & INIT_RCC)
		RCC_Configuration();
	if(iFlags & INIT_GPIO)
		GPIO_Configuration();
	if(iFlags & INIT_XMCLK)
		InitXMAPLEClk();
	if(iFlags & INIT_XMEXT)
		InitEXTINT();
	
	delay_init(SystemCoreClock / 1000000);
}

unsigned int isInterrupt(void)
{
    return (SCB->ICSR & SCB_ICSR_VECTACTIVE_Msk) != 0 ;
}

/*----------------------------------------------------------------------------
  Setup pool timer 
 *----------------------------------------------------------------------------*/

void RCC_POOL_Configuration(TIM_TypeDef* TIMx)
{
  /* TIM1 clock enable */
	if(TIMx == TIM1)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);
	else if(TIMx == TIM2)
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
	else if(TIMx == TIM3)
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
	else if(TIMx == TIM4)
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);
}

void POOL_TIMER_Init(TIM_TypeDef* TIMx)
{
	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
	uint16_t PrescalerValue = 0;

	RCC_POOL_Configuration(TIMx);
  /* Compute the prescaler value */
  PrescalerValue = (uint16_t) (SystemCoreClock / 1000) - 1;

  /* Time base configuration */
  TIM_TimeBaseStructure.TIM_Period = POOL_PERIOD_VALUE;
  TIM_TimeBaseStructure.TIM_Prescaler = PrescalerValue;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV2;
	TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIMx, &TIM_TimeBaseStructure);
}

void ChargePOOLTimer(TIM_TypeDef* TIMx, unsigned short wTime)
{
	TIMx->ARR = wTime;
	TIMx->CR1 = OPM | CEN;
}

unsigned int isTimeoutEnds(TIM_TypeDef* TIMx)
{
	return  (!(TIMx->CR1 & CEN));
}

/*************************************************************
**									SPECIAL BOARD FUNCTIONS									**
**************************************************************/

/*Detect XMAPLE CPLD Version*/
unsigned char GetXMVer(void)
{
	unsigned char bRetVer;
	unsigned char bRetDataA[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	unsigned char bRetDataB[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	int i, k;
	
	DisMIECOMP();
	SDCKA_OUT();
	SDCKB_OUT();
	
	for(i = 0;i < 8;i++)
	{
		SDCKB_SET(0);
		SDCKA_SET(PAT_DETA & (1 << i));

	//	bRetVer = RXMDState();
		
		for(k = 0;k < 8;k++)
			bRetDataA[k] |= (((RXMDState() >> k) & 0x01) << i);

		SDCKA_SET(0);
		SDCKB_SET(PAT_DETB & (1 << i));
		
		//bRetVer = RXMDState();
		
		for(k = 0;k < 8;k++)
			bRetDataB[k] |= (((RXMDState() >> k) & 0x01) << i);
	}
	
	k = 0;

	for(i = 0;i < 8;i++)
	{
		if(bRetDataA[i] == PAT_DETA)
		{
			bRetVer |= (i << 4);
			k |= 0x10;
		}
		
		if(bRetDataB[i] == PAT_DETB)
		{
			bRetVer |= (i);
			k |= 0x01;
		}
	}
	
	SDCKA_IN();
	SDCKB_IN();
	
	SDCKA_SET(1);
	SDCKB_SET(1);
	
	if(k != 0x11) /* Check Version Detection Mark */
		return 0x00;
	
	return bRetVer;
}
