#include <string.h>
#include "../MIELIB/inc/maplebus.h"

unsigned char FuncDeviceRequest(unsigned char APAddress)
{
	outMapleFrame.Command = DeviceRequest;
	outMapleFrame.DestAP = AP_PORTA | APAddress;
	outMapleFrame.OriginAP = AP_PORTA;
	outMapleFrame.DataSize = 0;

	while(MapleSend(&outMapleFrame, TIMEOUT_CHECK) != TRM_OK);

	if(isInterrupt())
		return MapleReceive(&inMapleFrame);
	
	return MapleReceive(&inMapleFrame);
}

/* Storrage access functions */
unsigned char GetStorageInfo(unsigned char APAddress, LPSTORAGE_INFO lpsSInf)
{
//	unsigned char *ptr = (unsigned char *)lpsSInf;
//	int i;
	unsigned char bMapleRet;
	
	outMapleFrame.Command = GetMediaInfo;
	outMapleFrame.DestAP	 = AP_PORTA | APAddress;
	outMapleFrame.OriginAP = AP_PORTA;
	outMapleFrame.DataSize = 2;

	/* Specifies the function type as "Storage" */
	outMapleFrame.Data.dwLine[0] = STORAGE;
	/* Specifies the partition number or LCD number */
	outMapleFrame.Data.dwLine[1] = MAKE_DWORD(0x00000000);

	while(MapleSend(&outMapleFrame, TIMEOUT_CHECK) != TRM_OK);
	
	if(isInterrupt())
		bMapleRet = MapleReceive(&inMapleFrame);
	else
		bMapleRet = MapleReceive(&inMapleFrame);

	if (bMapleRet == RCV_OK)
	{
		// Copy data as swapped
		/*
		for(i = 4;i < sizeof(STORAGE_INFO) + 4;i+=2)
		{
			*ptr++ = inMapleFrame.Data.bLine[i+1];
			*ptr++ = inMapleFrame.Data.bLine[i];
		}
		*/
		if(inMapleFrame.Command != DataTransfer)
			return inMapleFrame.Command;
		
		memcpy(lpsSInf, inMapleFrame.Data.bLine + 4, sizeof(STORAGE_INFO));
	}
			
	return bMapleRet;
}
/*
	OneBlockRead(
		unsigned char lpbBuffer,			//Pointer for received data buffer
		unsigned short wBlockNumber		//Block number for read
		)
	
	return: Receive Status
*/

unsigned short OneBlockRead(unsigned char APAddress, unsigned char *BlockData, unsigned short wBlockNumber, unsigned short wFuncType)
{
	unsigned char bMapleRet;
	/* Get Media Info */
	outMapleFrame.Command  = BlockRead;
	outMapleFrame.DestAP	 = AP_PORTA | APAddress;
	outMapleFrame.OriginAP = AP_PORTA;
	outMapleFrame.DataSize = 0x02;
	/* And Eddition data */
	outMapleFrame.Data.dwLine[0] = MAKE_DWORD(wFuncType);
	/* Specifies the partition number, always 0x00 */
	outMapleFrame.Data.bLine[0x4] = 0x00;
	/* Specifies the phase. Start from 0x00, next read 0x01 and etc...*/
	outMapleFrame.Data.bLine[0x5] = 0x00;
	outMapleFrame.Data.bLine[0x6] = (wBlockNumber >> 0x08) & 0xFF;
	outMapleFrame.Data.bLine[0x7] = wBlockNumber & 0xFF;
	/* Send prepared packet */
	while(MapleSend(&outMapleFrame, TIMEOUT_CHECK) != TRM_OK);
	/* Receive Answer */
	if(isInterrupt())
		bMapleRet = MapleReceive(&inMapleFrame);
	else
		bMapleRet = MapleReceive(&inMapleFrame);
	
	if(bMapleRet != RCV_OK && bMapleRet != RCV_CRC_ERROR)
		return 0x00000000;
	/*Check for device valid report*/
	if(inMapleFrame.Command != DataTransfer)
		return 0x00000000;
	/* Return Receive State*/
	memcpy(BlockData, inMapleFrame.Data.bLine + 0x08, (inMapleFrame.DataSize - 0x02) * 4);
	
	return (inMapleFrame.DataSize - 0x02) * 4;
}

/* Storrage access functions */

/* Write Large Block By 4 fases
	OneBlockWrite(
		unsigned short wBlockNumber		//Block number for write
		unsigned char lpbBuffer,			//Pointer for write data
		)
	
	return: Receive Status
*/

unsigned char FramesBlockWrite(unsigned char APAddress, unsigned short wBlockNumber, unsigned short wFuncType, unsigned char *lpbBuffer)
{
	unsigned char bRet;
	int iPhases = 4;
	int iBlockSize = 0x200;
	int i;
	/* Get Media Info */
	outMapleFrame.Command  = BlockWrite;
	outMapleFrame.DestAP	 = AP_PORTA | APAddress;
	outMapleFrame.OriginAP = AP_PORTA;
	/* And Eddition data */
	outMapleFrame.Data.dwLine[0] = MAKE_DWORD(wFuncType);
	/* Specifies the partition number, always 0x00 */
	outMapleFrame.Data.bLine[0x4] = 0x00;
	/* Specifies the phase. Start from 0x00, next write 0x01 and etc...*/
		outMapleFrame.Data.bLine[0x5] = 0x00;
	/* And block number for write */
	outMapleFrame.Data.bLine[0x6] = (wBlockNumber>>0x08) & 0xFF;
	outMapleFrame.Data.bLine[0x7] = wBlockNumber & 0xFF;
	/*Copy data to output buffer*/
	
	for(i = 0;i < iPhases;i ++)
	{
		/*2 + 512/Phases Bytes of data*/
		outMapleFrame.DataSize = (iBlockSize >> 2) / iPhases + 2;
		/*Set Phase Number*/
		outMapleFrame.Data.bLine[0x5] = i;
		/*Copy Data Part*/
		memcpy(&outMapleFrame.Data.bLine[8], lpbBuffer + i * ((iBlockSize) / iPhases), (iBlockSize) / iPhases);
		/* Send prepared packet */
		while(MapleSend(&outMapleFrame, TIMEOUT_CHECK) != TRM_OK);
		/* Receive Answer */
		if(isInterrupt())
			bRet = MapleReceive(&inMapleFrame);
		else
			bRet = MapleReceive(&inMapleFrame);
		/* Assign data with receiver buffer*/
		//lpbBuffer = inMapleFrame.Data.bLine;
	}
	
	/* Check For Writes Error */
	outMapleFrame.Command  = GetLastError;
	outMapleFrame.DestAP	 = AP_PORTA | APAddress;
	outMapleFrame.OriginAP = AP_PORTA;
	outMapleFrame.DataSize = 0x02;
	
		/*Set Phase Number 5*/
	outMapleFrame.Data.bLine[0x5] = i;
	/*Block number and partition come from previously defines*/
	
	while(MapleSend(&outMapleFrame, TIMEOUT_CHECK) != TRM_OK);
		/* Receive Answer */
	if(isInterrupt())
		bRet = MapleReceive(&inMapleFrame);
	else
		bRet = MapleReceive(&inMapleFrame);
		
	/* Return Receive State*/
	return bRet;
}

/*
	OneBlockWrite(
		unsigned short wBlockNumber		//Block number for write
		unsigned char lpbBuffer,			//Pointer for write data
		)
	
	return: Receive Status
*/

unsigned char OneBlockWrite(unsigned char APAddress, unsigned short wBlockNumber, unsigned short wFuncType, unsigned char *lpbBuffer, int iBlockSize)
{
	unsigned char bRet;
	int i = 0;
	/* Get Media Info */
	outMapleFrame.Command  = BlockWrite;
	outMapleFrame.DestAP	 = AP_PORTA | APAddress;
	outMapleFrame.OriginAP = AP_PORTA;
	/* And Eddition data */
	outMapleFrame.Data.dwLine[0] = MAKE_DWORD(wFuncType);
	/* Specifies the partition number, always 0x00 */
	outMapleFrame.Data.bLine[0x4] = 0x00;
	/* Specifies the phase. Start from 0x00, next write 0x01 and etc...*/
		outMapleFrame.Data.bLine[0x5] = 0x00;
	/* And block number for write */
	outMapleFrame.Data.bLine[0x6] = (wBlockNumber>>0x08) & 0xFF;
	outMapleFrame.Data.bLine[0x7] = wBlockNumber & 0xFF;
	/*Copy data to output buffer*/
	
		/*2 + 512/Phases Bytes of data*/
		outMapleFrame.DataSize = (iBlockSize >> 2);
		/*Set Phase Number*/
		outMapleFrame.Data.bLine[0x5] = i;
		/*Copy Data Part*/
		memcpy(&outMapleFrame.Data.bLine[8], lpbBuffer, iBlockSize);
		/* Send prepared packet */
		while(MapleSend(&outMapleFrame, TIMEOUT_CHECK) != TRM_OK);
		/* Receive Answer */
		if(isInterrupt())
			bRet = MapleReceive(&inMapleFrame);
		else
			bRet = MapleReceive(&inMapleFrame);
		/* Assign data with receiver buffer*/
		//lpbBuffer = inMapleFrame.Data.bLine;
	if(bRet)
		return bRet;
	/* Check For Writes Error */
	outMapleFrame.Command  = GetLastError;
	outMapleFrame.DestAP	 = AP_PORTA | APAddress;
	outMapleFrame.OriginAP = AP_PORTA;
	outMapleFrame.DataSize = 0x02;
	
		/*Set Phase Number 5*/
	outMapleFrame.Data.bLine[0x5] = i;
	/*Block number and partition come from previously defines*/
	
	while(MapleSend(&outMapleFrame, TIMEOUT_CHECK) != TRM_OK);
		/* Receive Answer */
	if(isInterrupt())
		bRet = MapleReceive(&inMapleFrame);
	else
		bRet = MapleReceive(&inMapleFrame);
		
	/* Return Receive State*/
	return bRet;
}

