/*----------------------------------------------------------------------------
 * Name:    interfaces.c
 * Purpose: MAPLE BUS condition report generate for some devices
 * Version: V1.00
 *----------------------------------------------------------------------------
 * This file is part of the OnAMAPLE tools.
 *
 * Copyright (c) 2007-2021 r.a.m. Factory STORE. All rights reserved.
 *----------------------------------------------------------------------------*/
 
#include "../inc/interfaces.h"

#ifdef MAPLE_DEVICE
/* GetControllerCondition
Report Joypad Condition
*/
unsigned char GetControllerCondition(LPCONTROLLERCONDITION ContCond)
{
//							X			Ra,La,Da,Ua,St,A,B	
//0b00000000	0b00000000	0b00000100	0b11111110	
	ContCond->ButtonsLO=0xFF;
	ContCond->ButtonsHI=0xFF;
	ContCond->AR_axis=0x00;//Left Lever not pushed
	ContCond->AL_axis=0x00;//Left Lever not pushed
	ContCond->Xa_axis=0x80;//center
	ContCond->Ya_axis=0x80;//center
	ContCond->Not_used_A=0x80;
	ContCond->Not_used_B=0x80;

	//Joypad Get Status
	#if defined (EN_CONTROLLER)
		extern void assigndata_cont(LPCONTROLLERCONDITION ContCond);
		assigndata_cont(ContCond);
	#endif

	return sizeof(CONTROLLERCONDITION)/4;
}

/* GetMouseCondition
Report Mouse Condition
*/
unsigned char GetMouseCondition(LPMOUSECONDITION MouseCond)
{
//press = '0', release = '1'.
	MouseCond->BTN=DEF_BUTTON|
		MW_BUTTON|
		MA_BUTTON|
		MB_BUTTON;	//Operating button data 
//no overflow=0
//AOV0
//AOV1
//AOV2
	MouseCond->AOV=0x0000;	//Analog coordinate overflow 
//
	MouseCond->AC1=0x0000;	//X-axis analog coordinate region
	MouseCond->AC2=0x0000;	//Y-axis analog coordinate region 
	MouseCond->AC3=0x0000;	//Z-axis analog coordinate region (wheel) 
	MouseCond->AC4=ST_CONSTANT;	//Constant (00h-02h, fixed) 
	MouseCond->AC5=ST_CONSTANT;	//Constant (00h-02h, fixed) 
	MouseCond->AC6=ST_CONSTANT;	//Constant (00h-02h, fixed) 
	MouseCond->AC7=ST_CONSTANT;	//Constant (00h-02h, fixed) 
	MouseCond->AC8=ST_CONSTANT;	//Constant (00h-02h, fixed) 

	//Joypad Get Status
	#if defined (EN_MOUSE)
		extern void assigndata_mouse(LPMOUSECONDITION MouseCond);
		assigndata_mouse(MouseCond);
	#endif
	
	return sizeof(MOUSECONDITION)/4;
}

/* GetKeyboardCondition
Report Keyboard Condition
*/
unsigned char GetKeyboardCondition(LPKEYBOARDCONDITION KeybCond)
{
	KeybCond->ChangeKeyBit=0x00;
	KeybCond->LEDInformation=0x00;
	
	//Keyboard Get Status
	#if defined (EN_KEYBOARD)
		#warning "No implement for Keyboard status function!"
	#endif

	return sizeof(KEYBOARDCONDITION)/4;
}

#endif // MAPLE_DEVICE
