 ;*----------------------------------------------------------------------------
 ;* Name:    miercv.s
 ;* Purpose: assembly file for Small MAPLE BUS logic receiver
 ;* Version: V1.00
 ;*----------------------------------------------------------------------------
 ;* This file is part of the OnAMAPLE tools.
 ;*
 ;* Copyright (c) 2007-2013 r.a.m. Factory STORE. All rights reserved.
 ;*----------------------------------------------------------------------------

CRL				EQU     0x00
CRH				EQU     0x04
IDR				EQU     0x08
ODR				EQU     0x0C
BSRR			EQU     0x10
BRR				EQU     0x14

;MIE Companion Signalling IO Port
MIE_PORT		EQU		0x40010800	;GPIO Port A 
;8 Bit Data port for receive
DATA_PORT		EQU		0x40011000	;GPIO Port C 
EXTI			EQU		0x40010400	;External Interrupt port base address
IMR				EQU		0x00
PR				EQU		0x14

				AREA    MIE_RCV, CODE, READONLY, ALIGN=2

_ReceiveMapleFrame PROC	;Negedge - Receive New Byte
                EXPORT  _ReceiveMapleFrame		[WEAK] ;11 Clocks
				IMPORT	inMaplePointer
				IMPORT	inMapleEndPointer
				IMPORT	m_bCRC
				IMPORT	m_bTimeOut
				IMPORT	m_bDWCounter
				IMPORT	m_bOVF
				IMPORT	m_dwTemp
;Stored register R0  R4 and R12				
				push	{R0-R4, R12}
				;Clear CRC Byte
				LDR		R1, = (m_bCRC)
				EOR		R2, R2
				STRB	R2, [R1]
				;Clear Timeout Byte
				LDR		R1, = (m_bTimeOut)
				STRB	R2, [R1]
				;Clear Overflow Byte
				LDR		R1, = (m_bOVF)
				STRB	R2, [R1]
				;Clear DW Counter
				LDR		R1, = (m_bDWCounter)
				STRB	R2, [R1]

				LDR		R4, = (MIE_PORT)
				LDR		R3, = (DATA_PORT)
				LDR		R1, = (36000) ; timeout
WaitPacketStart
;Wait RxD = 1 (transmit in progress)
				SUBS	R1, #1
				BEQ		TimeOut
				LDR		R2, [R4,#IDR]
				ANDS	R2,#0x08
				BEQ		WaitPacketStart
ContinueWaitOne
				LDR		R2, [R4,#IDR]
;if RxD = 0, transmition end
				TST		R2,#0x08
				BEQ		Exit
;wait for nDLatch = 1
				TST		R2,#0x01
				BEQ		ContinueWaitOne
ContinueWaitZero
				LDR		R2, [R4,#IDR]
				LDR		R1, [R3,#IDR]
;				TST		R2,#0x08
;				BEQ		Exit
;wait for nDLatch = 0
				TST		R2,#0x01
				BNE		ContinueWaitZero
				;Read data port from MIE COMPANION
				;Add pointer handler
				LDR		R2, = (inMaplePointer)
				LDR		R3, [R2]
				
				;CRC Check
				LDR		R0, = (m_bCRC)
				LDRB	R12, [R0]
				;Calc Current CRC
				EOR		R12, R1
				STRB	R12, [R0]
				
				;Store DWORD Data
				LDR		R12, = (m_dwTemp)
				STRB	R1, [R12]
				LDR		R1, [R12]
				;Check One Double Word counted
				LDR		R12, = (m_bDWCounter)
				LDRB	R0, [R12]
				AND		R0, #0xFF
				ADD		R0,#1

				CMPS	R0,#4
				BNE		NoDWCountedInt

				;Check Overflow
				LDR		R0, = (inMapleEndPointer)
				LDR		R0, [R0]
				CMPS	R3,R0
				BEQ		OverflowWhileDataReceiveInt

				EOR		R0,R0
				STR		R1, [R3]
				ADD		R3,#4
				STR		R3, [R2]
				B		DWCountedInt
NoDWCountedInt
				LSL		R1,#8
				LDR		R12, = (m_dwTemp)
				STR		R1, [R12]
DWCountedInt
				LDR		R12, = (m_bDWCounter)
				STRB	R0, [R12]

				LDR		R4, = (MIE_PORT)
				LDR		R3, = (DATA_PORT)
				B		ContinueWaitOne
Exit
				pop	{R0-R4, R12}
				BX      LR
OverflowWhileDataReceiveInt
				;Set overflow Byte
				LDR		R2, = (m_bOVF)
				MOV		R1, #0x01
				STRB	R1, [R2]
				
				pop	{R0-R4, R12}
				BX      LR
TimeOut				
				LDR		R2, = (m_bTimeOut)
				MOV		R1, #0x01
				STRB	R1, [R2]
				
				pop	{R0-R4, R12}
				BX      LR
                ENDP

; This assembly listing work with interrupts processing on lines SDCKA SDCKB

EXTI0_IRQHandler PROC	;Negedge - Receive New Byte
                EXPORT  EXTI0_IRQHandler		[WEAK] ;11 Clocks
				IMPORT	inMaplePointer
				IMPORT	inMapleEndPointer
				IMPORT	m_bCRC
				IMPORT	m_bDWCounter
				IMPORT	m_bOVF
				IMPORT	m_dwTemp
;Stored register by CORE while in INTERRUPT: R0  R3 and R12				
;				push	{R1}
				LDR		R1, = (DATA_PORT)
;				NOP
				LDR		R1, [R1,#IDR]
;				NOP
				
;				push	{R2-R5}
				
				;Clear Interrupt pending, by set PR Bit
				
				LDR		R3, = (EXTI)
				LDR		R2,[R3,#PR]
				ORR		R2,#0x01
				STR		R2,[R3,#PR]
				
				;Add pointer handler
				LDR		R2, = (inMaplePointer)
				LDR		R3, [R2]
				
				;CRC Check
				LDR		R0, = (m_bCRC)
				LDRB	R12, [R0]
				;Calc Current CRC
				EOR		R12, R1
				STRB	R12, [R0]
				
				;Store DWORD Data
				LDR		R12, = (m_dwTemp)
				STRB	R1, [R12]
				LDR		R1, [R12]
				;Check One Double Word counted
				LDR		R12, = (m_bDWCounter)
				LDRB	R0, [R12]
				AND		R0, #0xFF
				ADD		R0,#1

				CMPS	R0,#4
				BNE		NoDWCounted

				;Check Overflow
				LDR		R0, = (inMapleEndPointer)
				LDR		R0, [R0]
				CMPS	R3,R0
				BEQ		OverflowWhileDataReceive

				EOR		R0,R0
				STR		R1, [R3]
				ADD		R3,#4
				STR		R3, [R2]
				B		DWCounted
NoDWCounted				
				LSL		R1,#8
				LDR		R12, = (m_dwTemp)
				STR		R1, [R12]
DWCounted				
				LDR		R12, = (m_bDWCounter)
				STRB	R0, [R12]
				
;				pop		{R2-R5}
;				pop		{R1}
				BX      LR
OverflowWhileDataReceive	
				;Set overflow Byte
				LDR		R2, = (m_bOVF)
				MOV		R1, #0x01
				STRB	R1, [R2]
				
				LDR		R1, = (EXTI)
				LDR		R2,[R1,#IMR]
				AND		R2,#0xFFFFFFFC
				ORR		R3,#0x00000003
				STR		R3,[R1,#IMR]
				
;				pop		{R2-R5}
;				pop		{R1}
				BX      LR
                ENDP

EXTI1_IRQHandler PROC	;Posedge - Start Of Receive

                EXPORT  EXTI1_IRQHandler		[WEAK] ;11 Clocks
				IMPORT	inMapleFrame

;				push	{R0-R2}
				;Mask Receive Start, and unmask Receive Data
				LDR		R1, = (EXTI)
				LDR		R0,[R1,#IMR]
				AND		R0,#0xFFFFFFFC
				ORR		R0,#0x00000001
				STR		R0,[R1,#IMR]
				;Mask all pending, Include Data RCV Pending
				ORR		R0,#0x03
				STR		R0,[R1,#PR]
				;Clear CRC Byte
				LDR		R1, = (m_bCRC)
				EOR		R2, R2
				STRB	R2, [R1]
				;Clear Overflow Byte
				LDR		R1, = (m_bOVF)
				STRB	R2, [R1]
				;Clear DW Counter
				LDR		R1, = (m_bDWCounter)
				STRB	R2, [R1]
				;Setup Mapple pointer
				
;				LDR		R1, = (inMaplePointer)
;				LDR		R2, = (inMapleFrame)
;				STR		R2, [R1]
				;Check Overflow
				
				;And exit
;				pop		{R0-R2}
				BX      LR
                ENDP
				nop
	END
