/******************** (C) COPYRIGHT r.a.m Factory STORE ********************
* File Name          : maplebus.c
* Author             : r.a.m. Factory STORE
* Version            : V1.0.0
* Date               : 06/13/2008
* Description        : This file includes the peripherals header files in the
*                      user application.
********************************************************************************
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

#include "stm32f10x.h"

#include "../inc/xmapledma.h"
#include "../inc/Interfaces.h"
#include "../inc/Board.h"
#include "../inc/Delay.h"

/*Define Buffers For Mapple Transfer*/
MAPLEFRAME outMapleFrame;
MAPLEFRAME inMapleFrame;
/* Bytes counter for receiver */
volatile unsigned long m_dwTemp;

volatile unsigned char m_bRet = 0;

/*Calc CRC Byte, if it's not equal 0 after receive we have CRC Error*/
unsigned char m_bCRC;
/*Overflow Byte, if it's not equal 0 after receive we have Overflow Error*/
unsigned char m_bOVF;
/*Counter for byte receive*/
unsigned char m_bDWCounter;

unsigned char _bPoolTimerInited = 0x00;

#if !defined (MAPLE_HOST)
	#include "../lib/Devices.c"
#endif

#if defined (MAPLE_HOST)
	#include "../lib/Host.c"
	
/*Reset Pattern Send*/
void ResetDevice(void)
{
}
#endif

char *sRecErrorCodes[] = {	"Receive OK",
																		"CRC Detection error while receive data",
																		"Receive Buffer Overfflow",
																		"Timeout exceeded while Rx answer wait",
																		"Packet receive signalled, but START Pattern not detected",
																		"Frame Error"
																		};

extern void _mapleSend(LPMAPLEFRAME lpMapleFrame);
/*Maple send*/
unsigned char MapleSend(LPMAPLEFRAME lpMapleFrame, unsigned int boolTimeOutCheck)
{
	if(_bPoolTimerInited && boolTimeOutCheck == TIMEOUT_CHECK && !isTimeoutEnds(TIMEOUT_TIMER))
		return NO_TIMEOUT_EXCEEDED;
	
#if defined (MAPLE_HOST)
	SetupTransfer(&inMapleFrame);
#endif
	
	_mapleSend(lpMapleFrame);

#if defined (MAPLE_HOST)
	EnMIECOMP();
#endif
	
	if(!_bPoolTimerInited)
	{
		POOL_TIMER_Init(TIMEOUT_TIMER);
		_bPoolTimerInited = 0x01;
	}

	return TRM_OK;
}

/*Maple receive algorithm*/
/*
0. Disable ALL receive interrupts
1. Attach input Buffer
2. Enable receive start detect interrupt
*/
unsigned char MapleReceive(LPMAPLEFRAME MapleFrame)
{
	unsigned char bRet = RCV_OK;

#if defined (MAPLE_HOST)
	//Host must have timeouts
	unsigned int ulFrameTimeOut = RCV_TIME_OUT_CNT;
	/* Enable XMAPLECPLD receiver */
	
	/* Setup DMA Receiver */
/*	if(IsDMAEnds())
		SetupTransfer(&inMapleFrame);
	*/

	while(1)
	{																
			if(IsDMAEnds())
				break;

			ulFrameTimeOut--;
			delay_ms(1);
			
			if(!ulFrameTimeOut)	
			{
				//Receive ends with timeout
				bRet = RCV_TIME_OUT;
				DMAEnds();
				goto Exit;
			}

			// Frame Error
			if((GPIOA->IDR & FERR) || (!(m_wPortState & nSTRCV) && (m_wPortState & EOP)))
			{
				DMAEnds();
				bRet = RCV_FERR;
				goto Exit;
			}
	}																
	
		//Not Full Packet Received
		if(IsDMAEnds())
		{
			bRet = ValidateDMABuffer(MapleFrame);
		}
#else
	/* Enable XMAPLECPLD receiver */
	EnMIECOMP();
	/* Setup DMA Receiver */
	if(IsDMAEnds())
		SetupTransfer(&inMapleFrame);

#if defined (MOUSE_CALLBACK)
	extern void mouse_callback(void);
	mouse_callback();
#endif
	
	while(1)
	{
		// Frame Error
		if((GPIOA->IDR & FERR) || (!(m_wPortState & nSTRCV) && (m_wPortState & EOP)))
		{
			bRet = RCV_FERR;
			DMAEnds();
			break;
		}
		//No Receive
		if(IsDMAEnds())
		{
			bRet = ValidateDMABuffer(MapleFrame);
			break;
		}
	}
#endif
#pragma diag_suppress 177
Exit:
	DisableMapleINTsExceptRST();
	DisMIECOMP();
	if(_bPoolTimerInited)
		ChargePOOLTimer(TIMEOUT_TIMER, POOL_FRAME_TIMESHIFT);
	
	return m_bRet = bRet;
}
