/******************** (C) COPYRIGHT r.a.m Factory STORE ********************
* File Name          : xmapledma.c
* Author             : r.a.m. Factory STORE
* Version            : V1.0.0
* Date               : 06/13/2008
* Description        : This file includes dma on xmaple implement.
********************************************************************************
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

#include "stm32f10x.h"
#include <stm32f10x_rcc.h>
#include "../inc/xmapledma.h"

#include "../inc/Interfaces.h"
#include "../inc/Board.h"

unsigned char m_wPortState = 0x00;

/* nDLatch */
void EXTI0_IRQHandler(void)
{
	if (EXTI_GetITStatus(EXTI_Line0))
	{
		EXTI_ClearITPendingBit(EXTI_Line0);
		m_wPortState |= nDLatch;
	}
}

/* nSTRCV */
void EXTI1_IRQHandler(void)
{
	if (EXTI_GetITStatus(EXTI_Line1))
	{
		EXTI_ClearITPendingBit(EXTI_Line1);
		m_wPortState |= nSTRCV;
	}
}

/* nRST */
void EXTI2_IRQHandler(void)
{
	if (EXTI_GetITStatus(EXTI_Line2))
	{
		EXTI_ClearITPendingBit(EXTI_Line2);
		m_wPortState |= nRST_OCPY;
	}
}

/* RxD */
void EXTI3_IRQHandler(void)
{
	if (EXTI_GetITStatus(EXTI_Line3))
	{
		EXTI_ClearITPendingBit(EXTI_Line3);
		m_wPortState |= RxD;
	}
}

/* EOP */
void EXTI9_5_IRQHandler(void)
{
	if (EXTI_GetITStatus(EXTI_Line6))
	{
		EXTI_ClearITPendingBit(EXTI_Line6);
		DMAEnds();
		m_wPortState |= EOP;
	}
	else
	{
		void EXTI9_5_IRQHandlerCallback(void);
		EXTI9_5_IRQHandlerCallback();
	}
}

unsigned char ValidateDMABuffer(LPMAPLEFRAME MapleFrame)
{
	unsigned int uiDMASizeInBytes = sizeof(MAPLEFRAME) - DMA1_Channel5->CNDTR, ui, k;
	unsigned char *bPacket = (unsigned char *)MapleFrame;
	unsigned int *uiPacket = (unsigned int *)MapleFrame;
	
	if(uiDMASizeInBytes < 5) return false;
	
	/* Extract CRC Byte */
	unsigned char bCRC = bPacket[uiDMASizeInBytes - 1];
	/* Take Command Field Because we have rotated frame */
	unsigned int uiDataSize = MapleFrame->Command;

	if(uiDataSize > ((uiDMASizeInBytes - 5) / 4)) return RCV_OVF_ERROR;
	bPacket[uiDMASizeInBytes - 1]=0;
	
	for(ui = 0;ui <= uiDataSize;ui++)
	{
		for(k = 0;k < 4;k++)
		{
			bCRC ^= bPacket[(ui * 4) + k];
		}
		
		uiPacket[ui] = MAKE_DWORD(uiPacket[ui]);
	}
	
	if(bCRC) return RCV_CRC_ERROR;
	
	return RCV_OK;
}

void SetupTransfer(LPMAPLEFRAME MapleFrame)
{
//	static uint32_t DMACNTR = 0x0000;
   // Setup DMA.
   RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE); // Enable DMA1.
		DMA_DeInit(DMA1_Channel5);	//  TIM2_CH1
   {
        DMA_InitTypeDef Dma;
 
        Dma.DMA_MemoryBaseAddr = (uint32_t) MapleFrame;        // Data source DmaBuff array.
        Dma.DMA_PeripheralBaseAddr = (uint32_t) &(GPIOC->IDR); // Receiver - GPIOA port ODR register.
        Dma.DMA_DIR = DMA_DIR_PeripheralSRC;                   // Peripherals are data sinks.
        Dma.DMA_BufferSize = sizeof(MAPLEFRAME);               // The size of the data in the "words".
        Dma.DMA_PeripheralInc = DMA_PeripheralInc_Disable;     // Do not change the address of the receiver.
        Dma.DMA_MemoryInc = DMA_MemoryInc_Enable;              // Increase memory address.
        Dma.DMA_PeripheralDataSize = DMA_MemoryDataSize_Byte; // The size of the "word" - 2 bytes.
        Dma.DMA_MemoryDataSize = DMA_PeripheralDataSize_Byte;  // The size of the "word" - 2 bytes.
        Dma.DMA_Mode = DMA_Mode_Circular;                      // Cyclic work.
        Dma.DMA_Priority = DMA_Priority_Medium;                // DMA channel priority.
        Dma.DMA_M2M = DMA_M2M_Disable;                         // Do not copy from memory to memory.
        DMA_Init(DMA1_Channel5, &Dma);
   }
 
   // Timer setup.
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
	 TIM_DeInit(TIM2);
   {
        TIM_TimeBaseInitTypeDef Tim;
				TIM_ICInitTypeDef TIM_ICInitStructure;

        TIM_TimeBaseStructInit(&Tim);
		 
				Tim.TIM_ClockDivision = 0;
				Tim.TIM_CounterMode = TIM_CounterMode_Up;
				Tim.TIM_Period = 1;
				Tim.TIM_Prescaler = 0;
				Tim.TIM_RepetitionCounter = 0;
				TIM_TimeBaseInit(TIM2, &Tim);

				TIM_SelectInputTrigger(TIM2, TIM_TS_ETRF);
				TIM_SelectSlaveMode(TIM2, TIM_SlaveMode_External1);
		 
				TIM_ICInitStructure.TIM_Channel = TIM_Channel_1;
				TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Falling;
				TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
				TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
				TIM_ICInitStructure.TIM_ICFilter = 0;
				TIM_ICInit(TIM2, &TIM_ICInitStructure);
	 }
 
    TIM_GenerateEvent(TIM2, TIM_EventSource_CC1);
		TIM_SetAutoreload(TIM2, 1);
    TIM_DMACmd(TIM2, TIM_DMA_CC1, ENABLE);
    DMA_Cmd(DMA1_Channel5, ENABLE);
    TIM_Cmd(TIM2, ENABLE);
	 
		m_wPortState = 0;
	 
		EnableMaple_nSTRCV();
		EnableMapleRxD();
		EnableMaple_nEOP();
}
