#include "misc.h"
#include "ps_2.h"   


unsigned char PS2_Status = PS_2_CMDMODE;
unsigned char PS2_DATA_BUF[16];

unsigned char BIT_Count = 0;

void EXTI9_5_IRQHandlerCallback(void)
{
	static unsigned char tempdata = 0;
	static unsigned char parity = 0;
	if (EXTI->PR&(1 << SCL))
	{
		EXTI->PR = 1 << SCL;
		if (BIT_Count == 0)
		{
			parity = 0;
			tempdata = 0;
		}
		BIT_Count++;
		if (BIT_Count>1 && BIT_Count<10)
		{
			tempdata >>= 1;
			if (PS2_SDA)
			{
				tempdata |= 0x80;
				parity++;
			}
		}
		else if (BIT_Count == 10)
		{
			if (PS2_SDA)parity |= 0x80;
		}
		if (BIT_Count == 11)
		{
			BIT_Count = parity & 0x7f;
			if (((BIT_Count % 2 == 0) && (parity & 0x80)) || ((BIT_Count % 2 == 1) && (parity & 0x80) == 0))//????OK  
			{
				BIT_Count = PS2_Status & 0x0f;
				PS2_DATA_BUF[BIT_Count] = tempdata;
				if (BIT_Count<15)PS2_Status++;
				BIT_Count = PS2_Status & 0x30;
				switch (BIT_Count)
				{
				case PS_2_CMDMODE:
					PS2_Dis_Data_Report();
					PS2_Status |= 1 << 7;
					break;
				case PS_2_KEYBOARD:
					break;
				case PS_2_MOUSE:
					if (MOUSE_ID == 0)
					{
						if ((PS2_Status & 0x0f) == 3)
						{
							PS2_Status |= 1 << 7;
							PS2_Dis_Data_Report();
						}
					}
					else if (MOUSE_ID == 3)
					{
						if ((PS2_Status & 0x0f) == 4)
						{
							PS2_Status |= 1 << 7;
							PS2_Dis_Data_Report();
						}
					}
					break;
				}
			}
			else
			{
				PS2_Status |= 1 << 6;
				PS2_Status &= 0xf0;
			}
			BIT_Count = 0;
		}
	}
}

void PS2_Dis_Data_Report(void)
{
	PS2_Set_Int(0);
	PS2_SET_SCL_OUT();
	PS2_SCL_OUT = 0;
}

void PS2_En_Data_Report(void)
{
	PS2_SET_SCL_IN();
	PS2_SET_SDA_IN();
	PS2_SCL_OUT = 1;
	PS2_SDA_OUT = 1;
	PS2_Set_Int(1);
}

void PS2_Set_Int(unsigned char en)
{
	EXTI->PR = 1 << SCL;
	if (en)
		EXTI->IMR |= 1 << SCL;
	else
		EXTI->IMR &= ~(1 << SCL);
}

unsigned char Wait_PS2_Scl(unsigned char sta)
{
	u16 t = 0;
	sta = !sta;
	while (PS2_SCL == sta)
	{
		delay_us(1);
		t++;
		if (t > 1600) return 1;
	}
	return 0;
}

unsigned char PS2_Get_Byte(void)
{
	u16 t = 0;
	unsigned char temp = 0;
	while (1)
	{
		t++;
		delay_us(10);
		if (PS2_Status & 0x80)
		{
			temp = PS2_DATA_BUF[PS2_Status & 0x0f - 1];
			PS2_Status &= 0x70;
			break;
		}
		else if (t>100 || PS2_Status & 0x40)break;
	}
	PS2_En_Data_Report();
	return temp;
}

unsigned char PS2_Send_Cmd(unsigned char cmd)
{
	unsigned char i;
	unsigned char high = 0;
	PS2_Set_Int(0);
	PS2_SET_SCL_OUT();
	PS2_SET_SDA_OUT();
	PS2_SCL_OUT = 0;
	delay_us(120);
	PS2_SDA_OUT = 0;
	delay_us(10);
	PS2_SET_SCL_IN();
	PS2_SCL_OUT = 1;
	if (Wait_PS2_Scl(0) == 0)
	{
		for (i = 0; i<8; i++)
		{
			if (cmd & 0x01)
			{
				PS2_SDA_OUT = 1;
				high++;
			}
			else PS2_SDA_OUT = 0;
			cmd >>= 1;

			Wait_PS2_Scl(1);
			Wait_PS2_Scl(0);
		}
		if ((high % 2) == 0)PS2_SDA_OUT = 1;
		else PS2_SDA_OUT = 0;
		Wait_PS2_Scl(1);
		Wait_PS2_Scl(0);
		PS2_SDA_OUT = 1;
		Wait_PS2_Scl(1);
		PS2_SET_SDA_IN();
		Wait_PS2_Scl(0);
		if (PS2_SDA == 0)Wait_PS2_Scl(1);
		else
		{
			PS2_En_Data_Report();
			return 1;
		}
	}
	else
	{
		PS2_En_Data_Report();
		return 2;
	}
	PS2_En_Data_Report();
	return 0;
}

void MY_NVIC_PriorityGroupConfig(u8 NVIC_Group)
{
	u32 temp, temp1;
	temp1 = (~NVIC_Group) & 0x07;
	temp1 <<= 8;
	temp = SCB->AIRCR;
	temp &= 0X0000F8FF;
	temp |= 0X05FA0000;
	temp |= temp1;
	SCB->AIRCR = temp;
}

void MY_NVIC_Init(u8 NVIC_PreemptionPriority, u8 NVIC_SubPriority, u8 NVIC_Channel, u8 NVIC_Group)
{
	u32 temp;
	u8 IPRADDR = NVIC_Channel / 4;
	u8 IPROFFSET = NVIC_Channel % 4;
	IPROFFSET = IPROFFSET * 8 + 4;
	MY_NVIC_PriorityGroupConfig(NVIC_Group);
	temp = NVIC_PreemptionPriority << (4 - NVIC_Group);
	temp |= NVIC_SubPriority & (0x0f >> NVIC_Group);
	temp &= 0xf;

	if (NVIC_Channel<32)
		NVIC->ISER[0] |= 1 << NVIC_Channel;
	else
		NVIC->ISER[1] |= 1 << (NVIC_Channel - 32);

	NVIC->IP[IPRADDR] |= temp << IPROFFSET;
}

void PS2_Init(void)
{
	RCC->APB2ENR |= RCC_APB2ENR_IOPBEN;    //PORTB Enable

	GPIOB->CRH &= 0X00FFFFFF;		//PB6,7
	GPIOB->CRH |= 0X88000000;		//PB6,7
	GPIOB->ODR |= 3 << 24;

	AFIO->EXTICR[1] &= 0x0FFF;	//EXTI7 on PB7
	AFIO->EXTICR[1] = 0x1000;	//EXTI7 on PB7

	EXTI->IMR |= 1 << SCL;
	EXTI->EMR |= 1 << SCL;
	EXTI->FTSR |= 1 << SCL;
//	MY_NVIC_Init(1, 3, EXTI9_5_IRQChannel, 3);
}

